@@ -52,6 +52,7 @@ int
 	min_col_vpos,		/* lowest colums ending */
 	hpos,
 	h_offset = 0,		/* horizontal offset for multi-col output */
+	extra_space, rt_extra_space, rc_extra_space,	/* extra line spacing */
 	start_of_chorus,	/* Vertical positions of the chorus */
 	end_of_chorus,
 	chord_size, rt_chord_size, rc_chord_size,	/* font size for 'chord_font' */
@@ -89,7 +90,8 @@ int		/* BOOLEANS */
 	has_chord = FALSE,
 	title1_found = FALSE,
 	number_logical = FALSE,
-	debug_mode = FALSE;
+	debug_mode = FALSE,
+	in_order = TRUE;
 
 float
 	chord_inc,
@@ -113,6 +115,7 @@ t_pagespec
 	  { "letter", 792.0, 612.0 },
 	  { "us",     792.0, 612.0 },
 	  { "a4",     842.0, 595.0 },
+	  { "a5",     595.0, 421.0 },
 	};
 
 t_pagespec
@@ -316,29 +319,31 @@ char *command;
 	{
 	fprintf (stderr, "Usage: %s [options] file [file ...]\n", command);
 	fprintf (stderr, "Options:\n");
-	fprintf (stderr, "	-A                 : About Chordii...\n");
-	fprintf (stderr, "	-a                 : Automatic single space lines without chords\n");
-	fprintf (stderr, "	-c n               : Set chord size [9]\n");
-	fprintf (stderr, "	-C postscript_font : Set chord font\n");
-	fprintf (stderr, "	-D                 : Dumps chords definitions (PostScript)\n");
-	fprintf (stderr, "	-d                 : Dumps chords definitions (Text)\n");
-	fprintf (stderr, "	-G                 : Disable printing of chord grids\n");
-	fprintf (stderr, "	-g                 : Don't print grids for builtin \"easy\" chords.\n");
-	fprintf (stderr, "	-h                 : This message\n");
-	fprintf (stderr, "	-i                 : Generates a table of contents\n");
-	fprintf (stderr, "	-L                 : Even pages numbers on left\n");
-	fprintf (stderr, "	-l                 : Only print lyrics\n");
-	fprintf (stderr, "	-n                 : Number logical pages, not physical\n");
-	fprintf (stderr, "	-o filename        : Saves the output to file\n");
-	fprintf (stderr, "	-p n               : Starting page number [1]\n");
-	fprintf (stderr, "	-s n               : Set chord grid size [30]\n");
-	fprintf (stderr, "	-t n               : Set text size [12]\n");
-	fprintf (stderr, "	-T postscript_font : Set text font\n");
-	fprintf (stderr, "	-V                 : Print version and patchlevel\n");
-	fprintf (stderr, "	-x n               : Transpose by 'n' half-tones\n");
-	fprintf (stderr, "	-P type            : Specify page size [letter, a4 (default)]\n");
-	fprintf (stderr, "	-2                 : 2 pages per sheet\n");
-	fprintf (stderr, "	-4                 : 4 pages per sheet\n");
+	fprintf (stderr, "  --about  -A                   About Chordii...\n");
+	fprintf (stderr, "  --chord-font=FONT  -C         Sets chord font\n");
+	fprintf (stderr, "  --chord-grid-size=N  -s       Sets chord grid size [30]\n");
+	fprintf (stderr, "  --chord-grids-sorted  -S      Prints chord grids alphabetically\n");
+	fprintf (stderr, "  --chord-size=N  -c            Sets chord size [9]\n");
+	fprintf (stderr, "  --dump-chords  -D             Dumps chords definitions (PostScript)\n");
+	fprintf (stderr, "  --dump-chords-text  -d        Dumps chords definitions (Text)\n");
+	fprintf (stderr, "  --even-pages-number-left  -L  Even pages numbers on left\n");
+	fprintf (stderr, "  --help  -h                    This message\n");
+	fprintf (stderr, "  --lyrics-only  -l             Only prints lyrics\n");
+	fprintf (stderr, "  --no-chord-grids  -G          Disables printing of chord grids\n");
+	fprintf (stderr, "  --no-easy-chord-grids  -g     Doesn't print grids for builtin \"easy\" chords.\n");
+	fprintf (stderr, "  --output=FILE  -o             Saves the output to FILE\n");
+	fprintf (stderr, "  --page-number-logical  -n     Numbers logical pages, not physical\n");
+	fprintf (stderr, "  --page-size=FMT  -P           Specifies page size [letter, a4 (default)]\n");
+	fprintf (stderr, "  --single-space  -a            Automatic single space lines without chords\n");
+	fprintf (stderr, "  --start-page-number=N  -p     Starting page number [1]\n");
+	fprintf (stderr, "  --text-size=N  -t             Sets text size [12]\n");
+	fprintf (stderr, "  --text-font=FONT  -T          Sets text font\n");
+	fprintf (stderr, "  --toc  -i                     Generates a table of contents\n");
+	fprintf (stderr, "  --transpose=N  -x             Transposes by N semi-tones\n");
+	fprintf (stderr, "  --version  -V                 Prints Chordii version and exits\n");
+	fprintf (stderr, "  --vertical-space=N  -w        Extra vertical space between lines\n");
+	fprintf (stderr, "  --2-up  -2                    2 pages per sheet\n");
+	fprintf (stderr, "  --4-up  -4                    4 pages per sheet\n");
 
 	exit(0);
 	}
@@ -354,7 +359,7 @@ void do_about ()
 	printf("professional looking, impress-your-friends sheet-music suitable for printing\n");
 	printf("on your nearest PostScript printer.\n");
 	printf("\n");
-	printf("To learn more about Chordii, look for the man page or do \"chordii -h\" for\n");
+	printf("To learn more about Chordii, look for the man page or do \"chordii --help\" for\n");
 	printf("the list of options.\n");
 	printf("\n");
 	printf("			--0--\n");
@@ -451,7 +456,10 @@ char *chord;
 		{
 		if (strcmp(toupper_str(chord), NO_CHORD_STR))
 			{
-			ct_ptr = add_to_chordtab(chord);
+			if(in_order)
+				ct_ptr = add_in_order(chord);		
+			else
+				ct_ptr = add_to_chordtab(chord);
 			chord_line[j] = ct_ptr->chord->chord_name;
 			}
 		else	chord_line[j] = NO_CHORD_STR;
@@ -500,14 +508,19 @@ void init_ps()
 	{
 	printf ("%%!PS-Adobe-1.0\n");
 	printf ("%%%%Title: A nicely formatted song sheet\n");
-	printf ("%%%%Creator: %s %s.%s\n",
-		PACKAGE_NAME, PACKAGE_VERSION, PATCH_LEVEL);
+#ifdef WITH_LATIN2
+	printf ("%%%%Creator: %s %s latin2\n",
+		PACKAGE_NAME, PACKAGE_VERSION);
+#else
+ 	printf ("%%%%Creator: %s %s\n",
+ 		PACKAGE_NAME, PACKAGE_VERSION);
+#endif
 	printf ("%%%%DocumentFonts: (atend)\n");
 	printf ("%%%%Pages: (atend)\n");
 	printf ("%%%%BoundingBox: 5 5 %d %d\n", (int)(width-5), (int)(height-5));
 	printf ("%%%%EndComments\n");
 	printf ("/inch {72 mul } def\n");
-
+	
 	print_re_encode();
 	set_chord_font();
 	set_text_font(text_size);
@@ -685,6 +698,7 @@ void set_sys_def()
 	n_columns = 0;
 	max_columns = 1;
 	dummy_kcs.chord_name[0]='\0';
+	extra_space = DEF_EXTRA_SPACE;
 	}
 
 /* --------------------------------------------------------------------------------*/
@@ -697,6 +711,7 @@ void set_rc_def()
 	if (rc_text_font != NULL) text_font =  rc_text_font;
 	if (rc_chord_font != NULL) chord_font =  rc_chord_font;
 	if (rc_pagespec != NULL) pagespec = rc_pagespec;
+	if (rc_extra_space != 0) extra_space = rc_extra_space;
 	}
 
 /* --------------------------------------------------------------------------------*/
@@ -709,6 +724,7 @@ void set_rt_def()
 	if (rt_text_font != NULL) text_font =  rt_text_font;
 	if (rt_chord_font != NULL) chord_font =  rt_chord_font;
 	if (rt_pagespec != NULL) pagespec = rt_pagespec;
+	if (rt_extra_space != 0) extra_space = rt_extra_space;
 	}
 
 /* --------------------------------------------------------------------------------*/
@@ -772,6 +788,7 @@ int amount;
 void print_text_line()
 	{
 	int i;
+	int ns = extra_space;
 
 	text_line[i_text] = '\0';
 
@@ -781,7 +798,8 @@ void print_text_line()
 		{
 		advance(blank_space);
 		blank_space = 0;
-		advance (chord_size + 1);
+		advance (chord_size + 1 + ns);
+		ns = 0;
 
 		if ( ( text_line[i] != '\0' )
 		  && ( vpos - text_size <= bottom))
@@ -798,13 +816,13 @@ void print_text_line()
 
 	if ( text_line[i] == '\0')
 		{
-		blank_space += text_size - 2;
+		  blank_space += text_size - 2 + (in_tab ? 0 : extra_space);
 		}
 	else
 		{
 		advance (blank_space);
 		blank_space = 0;
-		advance (text_size - 1);
+		advance (text_size - 1 + (in_tab ? 0 : ns));
 		if (need_soc)
 			{
 			start_of_chorus = vpos + text_size;
@@ -1010,7 +1028,7 @@ char *directive;
 	else if (!strcmp(command, "columns") || !strcmp(command,"col"))
 		{
 		i = atoi(strtok(NULL, ": "));
-		if ( i <= 1 )
+		if ( i < 1 )
 			error ("invalid value for number of columns");
 		else
 			{
@@ -1148,8 +1166,14 @@ FILE *source_fd;
 				while (1) 
 					{
 					c = getc(source_fd);
-					if (c == '\n' || c == -1)
+					if (c == '\n' || c == EOF)
 						break;
+					if ( c == '\r' ) {
+					  if ( (c = getc(source_fd)) == EOF || c == '\n' )
+					    break;
+					  ungetc( c, source_fd );
+					  break;
+					}
 					}
 				i_input = 0;
 				do_directive(&directive[0]);
@@ -1162,6 +1186,15 @@ FILE *source_fd;
 				error("'}' found with no matching '{'");
 			break;
 
+		case '\r':
+		  if ( (c = getc(source_fd)) == EOF )
+		    goto got_eof;
+		  if ( c != '\n' ) {
+		    do_eol();
+		    ungetc( c, source_fd );
+		    break;
+		  }
+		  /* fall through */
 		case '\n':
 			do_eol();
 			break;
@@ -1202,10 +1235,7 @@ FILE *source_fd;
 		default :
 			if (in_chord )
 				{
-				if ( c != ' ' )
-					{
-					put_in_string(chord, &i_chord, c, CHORD_NAME_SZ, &i_chord_ov);
-					}
+				put_in_string(chord, &i_chord, c, CHORD_NAME_SZ, &i_chord_ov);
 				}
 			else if (in_directive)
 				{
@@ -1218,6 +1248,7 @@ FILE *source_fd;
 			break;
 			}
 		}
+got_eof:
 	if (i_input != 0 ) do_eol();
 	if (! in_chordrc) print_text_line();
 	}
@@ -1263,18 +1294,53 @@ void read_chordrc()
 
 
 /* --------------------------------------------------------------------------------*/
+
+static struct option long_options[] = {
+  { "2-up",		      no_argument,       0, '2' },
+  { "4-up",		      no_argument,       0, '4' },
+  { "about",		      no_argument,       0, 'A' },
+  { "chord-font",	      required_argument, 0, 'C' },
+  { "chord-grids-sorted",     no_argument,       0, 'S' },
+  { "chord-grid-size",	      required_argument, 0, 's' },
+  { "chord-size",	      required_argument, 0, 'c' },
+  { "dump-chords",	      no_argument,       0, 'D' },
+  { "dump-chords-text",	      no_argument,       0, 'd' },
+  { "even-pages-number-left", no_argument,       0, 'L' },
+  { "help",		      no_argument,       0, 'h' },
+  { "lyrics-only",	      no_argument,       0, 'l' },
+  { "no-chord-grids",	      no_argument,       0, 'G' },
+  { "no-easy-chord-grids",    no_argument,       0, 'g' },
+  { "output",		      required_argument, 0, 'o' },
+  { "page-number-logical",    no_argument,       0, 'n' },
+  { "page-size",	      required_argument, 0, 'P' },
+  { "single-space",	      no_argument,       0, 'a' },
+  { "start-page-number",      required_argument, 0, 'p' },
+  { "table-of-contents",      no_argument,       0, 'i' },
+  { "text-font",	      required_argument, 0, 'T' },
+  { "text-size",	      required_argument, 0, 't' },
+  { "toc",		      no_argument,       0, 'i' },
+  { "transpose",	      required_argument, 0, 'x' },
+  { "version",		      no_argument,       0, 'V' },
+  { "vertical-space",	      required_argument, 0, 'w' },
+  /* Do not forget to terminate the list, to prevent crashes */
+  /*  on unknown options. */
+  { NULL,                     0,                 0, 0   },
+};
+
 int main(argc, argv)
 int argc;
 char **argv;
 	{
 	int c,i;
+	int option_index = 0;
 
 	mesg = mesgbuf;
 /* Option Parsing */
 
 	command_name= argv[0];
-
-	while ((c = getopt(argc, argv, "aAc:C:dDgGhilLno:p:P:s:t:T:Vx:24")) != -1)
+	while ((c = getopt_long(argc, argv,
+				"aAc:C:dDgGhilLno:p:P:s:St:T:Vw:x:24",
+				long_options, &option_index)) != -1)
 		switch (c) {
 
 		case 'd':
@@ -1315,11 +1381,13 @@ char **argv;
 			break;
 
 		case 'x':
-			i = atoi (optarg);
-			if ( i == 0 )
+			{ char *p;
+			  i = strtol (optarg, &p, 10);
+			  if ( *p != '\0' || abs(i) > 12 )
 				error_rt("invalid value for transposition");
-			else
+			  else
 				transpose = i;
+			}
 			break;
 
 		case 's':
@@ -1329,6 +1397,10 @@ char **argv;
 			else
 				rt_grid_size = i;
 			break;
+		
+		case 'S':
+			in_order = FALSE;
+			break;
 
 		case 'g':
 			no_easy_grids = TRUE;
@@ -1402,16 +1474,15 @@ char **argv;
 			rt_pagespec = pagetype(optarg);
 			break;
 
+		case 'w':
+			rt_extra_space = atoi(optarg);
+			break;
+
 		case '?':
 			do_help(argv[0]);
 			break;
 		}
 
-/* Is there anything? */
-
-	if (argc == 1)
-		do_help(argv[0]);
-
 /* Is there input? */
 
 	if ((optind == argc) && isatty(0) && !dump_only)
@@ -1422,7 +1493,7 @@ char **argv;
 		fprintf (stderr, "Examples:\n");
 		fprintf (stderr, "   $ chordii my_song.cho > myfile.ps\n");
 		fprintf (stderr, "   $ chordii < mysong.cho > myfile.ps\n");
-		fprintf (stderr, "Do \"chordii -h\" to learn about Chordii's options\n");
+		fprintf (stderr, "Do \"chordii --help\" to learn about Chordii's options\n");
 		exit(1);
 	}
 
@@ -1431,10 +1502,13 @@ char **argv;
 	if (isatty(1) && (!dump_only || postscript_dump))  /* 1 == stdout  */
 	{
 		fprintf (stderr, "Error: Chordii will not send PostScript to your terminal.\n");
-		fprintf (stderr, "Please either redirect (>) or pipe (|) the output.\n");
+		fprintf (stderr, "Please redirect (>) or pipe (|) the output, or use\n");
+		fprintf (stderr, "the -o (--output) option to specify an output file.\n");
 		fprintf (stderr, "Examples:\n");
+		fprintf (stderr, "   $ chordii --output=myfile.ps my_song.cho\n");
 		fprintf (stderr, "   $ chordii my_song.cho > myfile.ps\n");
 		fprintf (stderr, "   $ chordii my_song.cho | lpr\n");
+		fprintf (stderr, "Do \"chordii --help\" to learn about Chordii's options\n");
 		exit(1);
 	}
 
@@ -1466,6 +1540,7 @@ char **argv;
 
 	if (do_toc)	/* generate index  page */
 		{
+		init_values();
 		build_ps_toc();
 		do_end_of_page(FALSE);
 		}
